package parser;

import genvisitor.*;
/**
 * Factory for a simple non-terminal that references a single token.
 */
public class SimpleFact extends ATVFactory {
    /**
     * Initializer lambda for this factory.
     */
    private ILambda _initializer = new ILambda() {
        public Object apply(Object param) {
            // change state to no-op
            _initializer = NoOpLambda.Singleton;

            // nothing to initialize
            return null;
        }
    };

    String _name;
    /**
     * Constructor for the F1 factory.
     *
     * @param tkz tokenizer to use
     * @param name the name that references the single token
     */
    public SimpleFact(String name, ITokenizer tkz) {
        super(tkz);
        _name = name;
    }

    private void addCmdTo(ATokVisitor<IGrammarToken, Object> target) {
      target.addCmd(_name, new AGenVisitor.IGenVisitorCmd<IGrammarToken, String, Object, AToken>() {
              public IGrammarToken apply(String idx, AToken host, Object... inp) {
                System.out.println("SimpleFact("+_name+")");
                return new SimpleToken(_name,(AToken)host);
              }
            });
    }
    
    /**
     * Make a token visitor to parse an E1a non-terminal.
     *
     * @return token visitor
     */
    public ATokVisitor<IGrammarToken, Object> makeVisitor() {
        initialize();

        return new ATokVisitor<IGrammarToken, Object>() {
          {
            addCmdTo(this);
          }
        };
    }

    /**
     * Make a token visitor that delegates to the given visitor in a chain of responsibility
     *
     * @param successor visitor to serve as successor in the chain
     */
    public ATokVisitor<IGrammarToken, Object> makeChainedVisitor(ATokVisitor<IGrammarToken, Object> successor) {
        initialize();

        return new ATokVisitor<IGrammarToken, Object>(successor) {
          {
            addCmdTo(this);
          }
        };
    }

    /**
     * Make the visitor.
     */
    private void initialize() {
        _initializer.apply(null);

    }
}

